      subroutine readpart(process)
c----------------------------------------------------------------------c
c Reads particles from process
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 9 February, 2006; modified 12 October, 2006 (particle reading improved)
c                            6 November, 2006 (wrong restoration of old
c                                              conventions removed),
c                            12 May, 2007 (mass and widths assignments
c                                          added)
c                            18 October, 2007 (MC summing over 
c                                              polarizations implemented)
c                            14 April, 2008 (determination of the 
c                            statistical factor for identical particles
c                            in the final state corrected)
c                            4 Feb., 2010 (information about generated 
c                            mappings printed)
c 28 February, 2012: short name for the process created automatically
c----------------------------------------------------------------------c

      use maxvals
      use mytypes
      use extpart
      use vertices
      use declends
      use modpart

      implicit none

      character(80) :: process,proc,cline

      character(1), parameter :: space=' ',separ='>'
      integer :: i,j,k,l,n,blad,iprt(mxprt,5),
     &           allmod,inmod,ii,difp,ifrmn,idscrd
      character(10) :: cprt(mxprt,4),ind1,ind2,ind3
      character(1) :: lprt(mxprt)

      integer, allocatable :: idpf(:)
      character(3), allocatable :: expd(:)
      character(9) :: prtcl(4)

      proc=adjustl(process)

      j=0
      l=index(proc,separ)

      if (l == 0) then
       write(*,*)'Wrong separator, or process not entered.'
       write(*,*)'Please, try again!'
       stop
      end if


c Read initial state particles

      do 
       n=index(proc,space)
       j=j+1
       expc(j)=proc(1:n-1)
       f(j)%o=-1
       proc=adjustl(proc(n:l))
       if (proc(2:2) == separ) exit
      end do

c No of the initial state particles

      nisp=j

c Read final state 

      k=l+2

      proc=adjustl(process(l+1:))

      do 
       n=index(proc,space)
       if (n > 1) then
        j=j+1
        f(j)%o=1
        expc(j)=proc(1:n-1)
       else
        exit
       end if
       proc=adjustl(proc(n:))
      end do

c No of the external particles

      next=j

c Create short name for the process

      if (trim(prcsnm) == 'auto') then
       prcsnm=''
       do j=1,nisp
        prcsnm=trim(adjustl(prcsnm))//trim(adjustl(expc(j)))
       end do
       prcsnm=trim(adjustl(prcsnm))//'_'
       do j=nisp+1,next
        prcsnm=trim(adjustl(prcsnm))//expc(j)(1:1)
       end do
      end if

c No of identical particles in the final state

      allocate (idpf(next-nisp),stat=blad)
      allocate (expd(next-nisp),stat=blad)

      idpf=1
      difp=1
      expd(difp)=expc(nisp+1)

      do j=nisp+2,next
       do ii=difp,1,-1
        if (expc(j) == expd(ii)) then
         idpf(ii)=idpf(ii)+1
         exit
        else
         if (ii == 1) then
          difp=difp+1
          expd(difp)=expc(j)
         end if
        end if
       end do
      end do

      nidp=1

      do j=1,difp
       do ii=1,idpf(j)
        nidp=nidp*ii
       end do
      end do

      deallocate (idpf,expd,stat=blad)

c      open(66,file='modpart.dat',access='direct',form='formatted',
c     &                                                     recl=80)
      open(unit=66,file='modpart.dat',status='old',action='read')

c Read comments

      do 
       read(66,'(a80)') cline
       if (index(cline,'Begin input') > 0 ) exit
      end do

c Read particles

      ipmod=0

      do 
       ipmod=ipmod+1
       read(66,'(4(a9),5(i5),a1)',end=600) cprt(ipmod,:),
     &                                    iprt(ipmod,:),lprt(ipmod)
c       ipmod=ipmod+1
      end do

 600  ipmod=ipmod-1

      close(66)

      do i=1,ipmod
       prtf(i)=iprt(i,1)
       prtg(i)=iprt(i,2)
       prtj(i)=iprt(i,5)
      end do

c Check if all the external particles belong to the model

      allmod=1

      do j=1,next
       inmod=0
       do i=1,ipmod
        if (trim(expc(j)) == trim(cprt(i,1))
     &     .or. trim(expc(j)) == trim(cprt(i,2))) then
         inmod=1
         exit
        end if
       end do
       allmod=allmod*inmod
       if (allmod == 0) then
        write(*,*) trim(expc(j)),' is not a particle of the model'
        stop 
       end if
      end do

      nmcs=0
      nhip=1

      do j=1,next
       f(j)%e=1
       allocate (f(j)%ix(f(j)%e),stat=blad)
       allocate (f(j)%ip(0),stat=blad)
       f(j)%ix(f(j)%e)=j
       do i=1,ipmod
        if (trim(expc(j)) == trim(cprt(i,1))
     &     .or. trim(expc(j)) == trim(cprt(i,2))) then
         expl(j)=lprt(i)
         if (trim(expc(j)) == trim(cprt(i,1))
     &      .and. trim(expc(j)) == trim(cprt(i,2))) then  ! neutral boson
          if (f(j)%o == -1) then
           f(j)%i=-1
          else
           f(j)%i=1
          end if
         else
          if (trim(expc(j)) == trim(cprt(i,1))) then
           f(j)%i=1
          else
           f(j)%i=-1
          end if
         end if
         f(j)%j=iprt(i,5)
         f(j)%f=iprt(i,1)
         f(j)%g=iprt(i,2)
         f(j)%n=iprt(i,4)
         f(j)%ct=iprt(i,3)
         f(j)%m=trim(cprt(i,3))
         f(j)%w=trim(cprt(i,4))
        end if
       end do
       if (f(j)%o == -1) then
        nhip=nhip*f(j)%n
       end if
       if (imcs == 1) then
        if (f(j)%n > 1) then
         nmcs=nmcs+1
         f(j)%mc=nmcs
         f(j)%n=1
        else
         f(j)%mc=0
        end if
       else
        f(j)%mc=0
       end if
      end do

c Define the SM vertices, using the notation
c f = 1-neutrino, 2-charged lepton, 3-up quark, 4-down quark;
c b = 0-Higgs, 1-photon, 2-Z, 3-W, 4-gluon.

c Read fermion-fermion-boson vertices of the SM and the corresponding couplings

      if (ickm == 1) then
       open(unit=66,file='vertices.dat.ckm',status='old',action='read')
      else
       open(unit=66,file='vertices.dat',status='old',action='read')
      end if

      nffba=0
      nbbba=0
      nbbbba=0

      do 
 1001  idscrd=0
       read(66,'(a80)',end=700) cline
       if (cline(1:1) == '#') then
       else
        cline=adjustl(cline)
        j=0
        l=index(cline,':')
        if (l == 0) then
         write(*,*)'Wrong separator in vertices.dat. It should be ":"'
         stop
        end if
        proc=cline(1:l-1)
        do 
         n=index(proc,space)
         if (n > 1) then
          j=j+1
          prtcl(j)=proc(1:n-1)
          if (iewk == 1) then
           do i=1,ipmod
            if (trim(prtcl(j)) == trim(cprt(i,1))) then
             if (iprt(i,5) == 21) then
              idscrd=1
              exit
             end if
            end if
           end do
          end if
          if (ihgs == 1) then
           do i=1,ipmod
            if (trim(prtcl(j)) == trim(cprt(i,1))) then
             if (iprt(i,5) == 25) then
              idscrd=1
              exit
             end if
            end if
           end do
          end if
         else
          exit
         end if
         if (idscrd == 1) go to 1001
         proc=adjustl(proc(n:))
        end do
        if (j == 4) then
         nbbbba=nbbbba+1
         if (nbbbba > nbbbb) then
          write(*,*)'Too many quartic couplings in vertices.dat.'//
     &              ' Increase nbbbb in maxvals.f.'
         end if
         do k=1,4
          do i=1,ipmod
           if (trim(prtcl(k)) == trim(cprt(i,1))) then
            bbbb(nbbbba,k)=iprt(i,5)
            exit
           else if (trim(prtcl(k)) == trim(cprt(i,2))) then
            bbbb(nbbbba,k)=-iprt(i,5)
            exit
           else
            if (i == ipmod) then
             write(*,*)'readpart.f: No such particle in modpart.f'
             stop
            end if
           end if
          end do
         end do
         gbbbb(nbbbba)=trim(adjustl(cline(l+1:)))
        else if (j == 3) then
         ifrmn=0
         do k=1,3
          do i=1,ipmod
           if (trim(prtcl(k)) == trim(cprt(i,1))
     &         .or. trim(prtcl(k)) == trim(cprt(i,2))) then
            if (iprt(i,2) /= 0) then
             ifrmn=1
             exit
            end if
           end if
          end do
          if (ifrmn == 1) exit
         end do
         if (ifrmn == 1) then
          nffba=nffba+1
          if (nffba > nffb) then
           write(*,*)'Too many fermion-fermion-boson couplings'//
     &               ' in vertices.dat.'
           write(*,*)'Increase nffb in maxvals.f.'
          end if
          do k=1,3
           do i=1,ipmod
            if (trim(prtcl(k)) == trim(cprt(i,1))) then
             ffb(nffba,k)=iprt(i,5)
             exit
            else if (trim(prtcl(k)) == trim(cprt(i,2))) then
             ffb(nffba,k)=-iprt(i,5)
             exit
            else
             if (i == ipmod) then
              write(*,*)'readpart.f: No such particle in modpart.f'
              stop
             end if
            end if
           end do
          end do
          gffb(nffba)=trim(adjustl(cline(l+1:)))
         else
          nbbba=nbbba+1
         if (nbbba > nbbb) then
          write(*,*)'Too many triple-boson couplings in vertices.dat.'
          write(*,*)'Increase nbbb in maxvals.f.'
         end if
          do k=1,3
           do i=1,ipmod
            if (trim(prtcl(k)) == trim(cprt(i,1))) then
             bbb(nbbba,k)=iprt(i,5)
             exit
            else if (trim(prtcl(k)) == trim(cprt(i,2))) then
             bbb(nbbba,k)=-iprt(i,5)
             exit
            else
             if (i == ipmod) then
              write(*,*)'readpart.f: No such particle in modpart.f'
              stop
             end if
            end if
           end do
          end do
          gbbb(nbbba)=trim(adjustl(cline(l+1:)))
         end if
        else
         write(*,*)'readpart.f: Only triple and quartic couplings'//
     &             ' are possible'
         stop
        end if
       end if
      end do

 700  close(66)

c      do i=1,nffba
c       write(*,*)'ffb(',i,':)=(/',ffb(i,:),'/)'
c       write(*,*)'gffb(',i,')=',gffb(i)
c      end do
c      do i=1,nbbba
c       write(*,*)'bbb(',i,':)=(/',bbb(i,:),'/)'
c       write(*,*)'gbbb(',i,')=',gbbb(i)
c      end do
c      do i=1,nbbbba
c       write(*,*)'bbbb(',i,':)=(/',bbbb(i,:),'/)'
c       write(*,*)'gbbbb(',i,')=',gbbbb(i)
c      end do

c      if (iewk == 1) then ! pure EW contributions
c       ffb(1:6,:)=0
c       bbb(1,:)=0
c       bbbb(1,:)=0
c      end if

      do i=1,nffba
       do j=1,3
        do k=1,ipmod
         if (abs(ffb(i,j)) == iprt(k,5)) then
          cffb(i,j)=iprt(k,3)
          mffb(i,j)=cprt(k,3)
          wffb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do

      do i=1,nbbba
       do j=1,3
        do k=1,ipmod
         if (abs(bbb(i,j)) == iprt(k,5)) then
          cbbb(i,j)=iprt(k,3)
          mbbb(i,j)=cprt(k,3)
          wbbb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do

      do i=1,nbbbba
       do j=1,4
        do k=1,ipmod
         if (abs(bbbb(i,j)) == iprt(k,5)) then
          cbbbb(i,j)=iprt(k,3)
          mbbbb(i,j)=cprt(k,3)
          wbbbb(i,j)=cprt(k,4)
          exit
         end if
        end do
       end do
      end do




      nhel=1
c      nhip=1
      ncip=1

      do j=1,next
       nhel=nhel*f(j)%n
       if (f(j)%o == -1) then
c        nhip=nhip*f(j)%n
        ncip=ncip*f(j)%ct
       end if
      end do

      open(16,file='partcls.tmp',access='direct',form='formatted',
     &                                                     recl=80)
c      open(20,file='orders.tmp',access='direct',form='formatted',
c     &                                                     recl=80)

      do j=1,next
       write(16,1612,rec=j) f(j)%j,f(j)%i,f(j)%o,f(j)%g,f(j)%f,f(j)%ct,
     &                      f(j)%n,f(j)%e
c       write(20,1620,rec=j) j
      end do
c 1612 format(6(i3),i24,i3)
 1612 format(i8,5(i3),i24,i3)
 1620 format(26(i3))

      close(16)
c      close(20)

      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      write(*,'(a)') 'Attempting the process'
      write(*,'(a)') ' '
      write(*,'(a)') '        '//trim(adjustl(process))
      write(*,'(a)') ' '
      write(*,'(a)') 'with '//trim(adjustl(prvrsn))//', in a model with'
c      write(ind1,'(i10)') ipmod
      write(ind1,'(i10)') nffba
      write(ind2,'(i10)') nbbba
      write(ind3,'(i10)') nbbbba
      write(*,'(a)') trim(adjustl(ind1))//' fermion-fermion-boson, '// 
     & trim(adjustl(ind2))//' triple-boson and '
     & //trim(adjustl(ind3))//' quartic-boson couplings,'
      write(*,'(a)') 'see a file ''vertices.dat'' for details.'
      write(*,'(a)') 'Implemented particles are listed in a file'
     & //                    '''modpart.dat''.'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'

      if (iewk == 1) then ! pure EW contributions
       write(*,'(a)') 'taking into account pure EW contributions'
      write(*,'(a)') '---------------------------------------------'//
     &  '--------------------'
      end if
      if (ihgs == 1) then ! No Higgs boson contributions
       write(*,'(a)') 'without the Higgs boson contributions'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (iwtb == 1) then ! Include the anomalous wtb coupling'
       write(*,'(a)') 'The anomalous Wtb coupling included'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      if (imcs == 1) then
       write(*,'(a)') 'MC summing over polarizations implemented'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'
      end if
      write(*,'(a,i4,a)') 'Propagators of subsystems with up to',mxmap,
     &                    ' particles will be mapped'
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'

      write(*,'(a)') 'External particles:'
      do i=1,next
       write(*,1) f(i)%ix,expc(i),f(i)%j,f(i)%i,f(i)%o,
     &            f(i)%o*f(i)%i*f(i)%j,f(i)%n
      end do
 1    format(i4,'  ',a3,' :',i8,2(i3),i8,i3)
      write(*,'(a)') '---------------------------------------------'//
     &  '-------------------------'

      end
