      subroutine couplsm(e,cmw,cmh)
c----------------------------------------------------------------------c
c Returns the standard model couplings
c 
c Input: e - electric charge,
c        cmw - complex W boson mass
c        cmh - complex Higgs boson mass
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 18 June, 2001; modified on 20 September, 2001; 23 June, 2005;
c                            28 March, 2006; 13 May, 2007
c----------------------------------------------------------------------c

      use inprms
      use couplsmkk
      use topan

      implicit none

      real(dp) :: e
      complex(dp) :: cmw,cmh

      complex(dp) :: sw,ccw2,cw,hff(2)

      sw=sqrt(csw2)
      ccw2=1.d0-csw2
      cw=sqrt(ccw2)

c Photon couplings to fermions:

      ann=ocom
      all=-cmplx(-e,0.d0,dp)
      auu=-cmplx(2.d0/3.d0*e,0.d0,dp)
      add=-cmplx(-1.d0/3.d0*e,0.d0,dp)

c Z boson couplings to fermions:

      znn=(/ocom,1.d0/(2.d0*sw*cw)*e/)
      zll=(/sw/cw*e,-(1.d0-2.d0*csw2)/(2.d0*sw*cw)*e/)
      zuu=(/-2.d0/3.d0*sw/cw*e,
     &                  (1.d0-4.d0/3.d0*csw2)/(2.d0*sw*cw)*e/)
      zdd=(/sw/cw/3.d0*e,-(1.d0-2.d0/3.d0*csw2)/(2.d0*sw*cw)*e/)

c W boson coupling to fermions:

      wff=(/ocom,e/sqrt(2.d0)/sw/)

c CKM mixing (real), if included:

      wud=wff*V_ud
      wus=wff*V_us
      wub=wff*V_ub
      wcd=wff*V_cd
      wcs=wff*V_cs
      wcb=wff*V_cb
      wtd=wff*V_td
      wts=wff*V_ts

c The Higgs boson coupling to fermions:

      hff=-e/(2.d0*sw*cmw)

      hee=sqrt(cmplx(me**2,-me*game,dp))*hff
      hmm=sqrt(cmplx(mmu**2,-mmu*gammu,dp))*hff
      hll=sqrt(cmplx(mtau**2,-mtau*gamtau,dp))*hff
      huu=sqrt(cmplx(mu**2,-mu*gamu,dp))*hff
      hdd=sqrt(cmplx(md**2,-md*gamd,dp))*hff
      hcc=sqrt(cmplx(mc**2,-mc*gamc,dp))*hff
      hss=sqrt(cmplx(ms**2,-ms*gams,dp))*hff
      htt=sqrt(cmplx(mt**2,-mt*gamt,dp))*hff
      if (ihtt == 1) then
       htt=htt*(/cmplx(httv,htta,dp),cmplx(httv,-htta,dp)/)
      end if
      hbb=sqrt(cmplx(mb**2,-mb*gamb,dp))*hff

c Gluon coupling to quarks:

      gqq=(/cmplx(es,0.d0,dp),cmplx(es,0.d0,dp)/)

c Triple gauge boson couplings:

      aww=cmplx(e,0.d0,dp)
      zww=-cw/sw*e
      ggg=-gqq(1)

c Quartic gauge boson couplings:

      aaww=cmplx(e**2,0.d0,dp)
      azww=-cw/sw*e**2
      zzww=ccw2/csw2*e**2
      wwww=-1.d0/csw2*e**2
      gggg=gqq(1)**2

c Higgs boson couplings to Z and W bosons:

      hzz=e*cmw/sw/ccw2
      hww=e*cmw/sw

c The triple Higgs boson coupling

      hhh=-3.d0*cmh**2/(2.d0*sw*cmw)*e

c Quartic Higgs boson couplings

      hhww=e**2/(2.d0*csw2)
      hhzz=e**2/(2.d0*csw2*ccw2)
      hhhh=-3.d0*cmh**2/(4.d0*csw2*cmw**2)*e**2


c Triple photon-pion coupling of scalar QED:

      app=cmplx(e,0.d0,dp)

c Quartic photon-pion coupling of scalar QED:

      aapp=cmplx(2.d0*e**2,0.d0,dp)


c Scalar-fermion couplings:

c      csff=e/(sqrt(2.d0)*sw*cmw)
c      c3ff=cmplx(0.d0,-e,dp)/(2.d0*sw*cmw)

c Scalar-gauge boson couplings:

c      csaw=e*cmw
c      cszw=-e*cmw*sw/cw 

c Outgoing phi-(p1), incoming phi-(p2):

c      cass=-e
c      czss=e*(sw**2-cw**2)/(2.d0*sw*cw)

c Outgoing phi3(p1), incoming phi-(p2):

c      cw3s=cmplx(0.d0,e,dp)/(2.d0*sw)

c Outgoing H(p1), incoming phi-(p2):

c      cwhs=-e/(2.d0*sw)

c Outgoing phi3(p1), incoming H(p2):

c      cz3h=cmplx(0.d0,e,dp)/(2.d0*sw*cw)

c Triple scalar couplings:

c      chss=-e*cmh**2/(2.d0*sw*cmw)

      end


